#!/usr/bin/env python

"""
Asynchronous Error Test
"""

import time
import socket
import zmq
from psdaq.control.control import back_pull_port, error_msg, warning_msg
import argparse

class Client:
    def __init__(self, args):

        # configure zmq sockets
        self.context = zmq.Context(1)
        self.push = self.context.socket(zmq.PUSH)
        self.push.connect('tcp://%s:%d' % (args.C, back_pull_port(args.p)))

        if args.error is not None:
            # send error message
            self.push.send_json(error_msg("%s: %s" % (args.alias, args.error)))

        if args.warning is not None:
            # send warning message
            self.push.send_json(warning_msg("%s: %s" % (args.alias, args.warning)))

def main():

    try:
        # process arguments
        parser = argparse.ArgumentParser()
        parser.add_argument('-p', type=int, choices=range(0, 8), default=0, help='platform (default 0)')
        parser.add_argument('-C', metavar='COLLECT_HOST', default='localhost', help='collection host (default localhost)')
        parser.add_argument('--error', metavar='ERR_MSG')
        parser.add_argument('--warning', metavar='WARN_MSG')
        parser.add_argument('--alias', metavar='ALIAS', required=True)
        args = parser.parse_args()

        if args.error is None and args.warning is None:
            parser.error('Must specify --error or --warning or both')

        # start client
        client = Client(args)

    except KeyboardInterrupt:
        print('KeyboardInterrupt')

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
selectPlatform command
"""
from psdaq.control.DaqControl import DaqControl
from psdaq.control.control import detector_name
import pprint
import argparse

def common_match(alias, arglist):
    return arglist is not None and alias in arglist

def drp_match(alias, arglist):
    if arglist is not None:
        for arg in arglist:
            if (arg == alias) or (arg == detector_name(alias)):
                return True
    return False

def main():

    # Process arguments
    parser = argparse.ArgumentParser(epilog='For multisegment detector, specify drp alias without _N suffix.')
    parser.add_argument('-p', metavar='PLATFORM', type=int, choices=range(0, 8), default=0, help='platform (default 0)')
    parser.add_argument('-C', metavar='COLLECT_HOST', default='localhost', help='collection host (default localhost)')
    parser.add_argument('-t', type=int, metavar='TIMEOUT', default=2000,
                        help='timeout msec (default 2000)')
    parser.add_argument('-R', metavar='READOUT_GROUP', type=int, choices=range(0, 8), help='readout group (0-7, default platform)')
    parser.add_argument('-s', metavar='SELECT', action='append', help='select one alias (may be repeated)')
    parser.add_argument('--select-all', action='store_true', help='select all', dest='select_all')
    parser.add_argument('-u', metavar='UNSELECT', action='append', help='unselect one alias (may be repeated)')
    args = parser.parse_args()

    platform = args.p
    if args.R is None:
        readout = platform
    else:
        readout = args.R

    # instantiate DaqControl object
    control = DaqControl(host=args.C, platform=args.p, timeout=args.t)

    try:
        body = control.getPlatform()
    except Exception as ex:
        print('getPlatform(): %s' % ex)
    except KeyboardInterrupt:
        pass
    else:
        changed = False
        try:
            for level in body:
                for k, v in body[level].items():
                    alias = v['proc_info']['alias']

                    # select ...
                    if level == 'drp':
                        if args.select_all or drp_match(alias, args.s):
                            if v['active'] != 1 or v['det_info']['readout'] != readout:
                                changed = True
                                v['active'] = 1
                                v['det_info']['readout'] = readout
                    else:
                        if args.select_all or common_match(alias, args.s):
                            if v['active'] != 1:
                                changed = True
                                v['active'] = 1

                    # unselect ...
                    match = False
                    if level == 'drp':
                        if drp_match(alias, args.u):
                            match = True
                    else:
                        if common_match(alias, args.u):
                            match = True
                    if match:
                        if v['active'] != 0:
                            changed = True
                            v['active'] = 0
        except Exception:
            pprint.pprint(body)
            raise
        else:
            if changed:
                try:
                    retval = control.selectPlatform(body)
                except Exception as ex:
                    print('selectPlatform(): %s' % ex)
                except KeyboardInterrupt:
                    pass
                else:
                    if 'err_info' in retval:
                        print('Error: %s' % retval['err_info'])

if __name__ == '__main__':
    main()



from psdaq.control.config_scan import scan
from psdaq.configdb.get_config import *
import numpy as np
import sys

detName  = 'epixquad_0'

def steps():
    d = {}
    metad = {}
    metad['detname'] = detName
    metad['scantype'] = 'pedestal'
    for gain in range(5):
        #  Set the detector level config change
        d[f'{detName}:user.gain_mode'] = gain
        #  Set the global meta data
        metad['step'] = gain
        yield (d, float(gain), json.dumps(metad))

if __name__ == '__main__':

    # default command line arguments
    if len(sys.argv)==1:
        defargs = ['-B','DAQ:UED','-p','0','-x','0','-C','drp-ued-cmp002','-c 1000','--config','BEAM']
        sys.argv.extend(defargs)

    keys = []
    keys.append(f'{detName}:user.gain_mode')

    scan(keys, steps)



import sys
import logging
import dgramCreate as dc
import threading
import zmq
import json

from psdaq.control.control import DaqControl, DaqPVA
import argparse

class MyDAQ:
    def __init__(self, control, *, daqState, args):
        self.zmq_port = 5550+args.p
        self.control = control
        self.name = 'mydaq'
        self.parent = None
        self.context = zmq.Context()
        self.push_socket = self.context.socket(zmq.PUSH)
        self.push_socket.bind('tcp://*:%s' % self.zmq_port)
        self.pull_socket = self.context.socket(zmq.PULL)
        self.pull_socket.connect('tcp://localhost:%s' % self.zmq_port)
        self.comm_thread = threading.Thread(target=self.daq_communicator_thread, args=())
        self.mon_thread = threading.Thread(target=self.daq_monitor_thread, args=(), daemon=True)
        self.ready = threading.Event()
        self.daqState = daqState
        self.args = args
        self.daqState_cv = threading.Condition()
        self.stepDone_cv = threading.Condition()
        self.stepDone = 0
        self.comm_thread.start()
        self.mon_thread.start()
        self.verbose = args.v
        self.pv_base = args.B

        if args.g is None:
            self.groupMask = 1 << args.p
        else:
            self.groupMask = args.g

        # StepEnd is a cumulative count
        self.readoutCount = args.c
        self.readoutCumulative = 0

        # instantiate DaqPVA object
        self.pva = DaqPVA(platform=args.p, xpm_master=args.x, pv_base=args.B, report_error=self.report_error)

    # this thread tells the daq to do a step and waits for the completion
    def daq_communicator_thread(self):
        logging.debug('*** daq_communicator_thread')
        while True:
            sss = self.pull_socket.recv().decode("utf-8")
            if ',' in sss:
                state, phase1 = sss.split(',', maxsplit=1)
            else:
                state, phase1 = sss, None

            logging.debug('*** received %s' % state)
            if state in ('connected', 'starting'):
                # send 'daqstate(state)' and wait for complete
                if phase1 is None:
                    errMsg = self.control.setState(state)
                else:
                    errMsg = self.control.setState(state, json.loads(phase1))

                if errMsg is not None:
                    logging.error('%s' % errMsg)
                    continue

                with self.daqState_cv:
                    while self.daqState != state:
                        logging.debug('daqState \'%s\', waiting for \'%s\'...' % (self.daqState, state))
                        self.daqState_cv.wait(1.0)
                    logging.debug('daqState \'%s\'' % self.daqState)

                self.ready.set()

            elif state=='running':
                # launch the step with 'daqstate(running)' (with the
                # scan values for the daq to record to xtc2).

                # set EPICS PVs.
                # StepEnd is a cumulative count.
                self.readoutCumulative += self.readoutCount
                self.pva.pv_put(self.pva.pvStepEnd, self.readoutCumulative)
                self.pva.step_groups(mask=self.groupMask)
                self.pva.pv_put(self.pva.pvStepDone, 0)
                with self.stepDone_cv:
                    self.stepDone = 0
                    self.stepDone_cv.notify()

                # set DAQ state
                if phase1 is None:
                    errMsg = self.control.setState(state)
                else:
                    errMsg = self.control.setState(state, json.loads(phase1))
                if errMsg is not None:
                    logging.error('%s' % errMsg)
                    continue

                with self.daqState_cv:
                    while self.daqState != 'running':
                        logging.debug('daqState \'%s\', waiting for \'running\'...' % self.daqState)
                        self.daqState_cv.wait(1.0)
                    logging.debug('daqState \'%s\'' % self.daqState)

                # define nested function for monitoring the StepDone PV
                def callback(stepDone):
                    with self.stepDone_cv:
                        self.stepDone = int(stepDone)
                        self.stepDone_cv.notify()

                # start monitoring the StepDone PV
                sub = self.pva.monitor_StepDone(callback=callback)

                with self.stepDone_cv:
                    while self.stepDone != 1:
                        logging.debug('PV \'StepDone\' is %d, waiting for 1...' % self.stepDone)
                        self.stepDone_cv.wait(1.0)
                logging.debug('PV \'StepDone\' is %d' % self.stepDone)

                # stop monitoring the StepDone PV
                sub.close()

                self.ready.set()

            elif state=='shutdown':
                break

    def daq_monitor_thread(self):
        logging.debug('*** daq_monitor_thread')
        while True:
            part1, part2, part3, part4, part5, part6, part7, part8 = self.control.monitorStatus()
            if part1 is None:
                break
            elif part1 not in DaqControl.transitions:
                continue

            # part1=transition, part2=state, part3=config
            with self.daqState_cv:
                self.daqState = part2
                self.daqState_cv.notify()

    def _set_connected(self):
        self.push_socket.send_string('connected')
        # wait for complete
        self.ready.wait()
        self.ready.clear()

    def stage(self):
        # done once at start of scan
        # put the daq into the right state ('connected')
        logging.debug('*** here in stage')
        self._set_connected()

    def unstage(self):
        # done once at end of scan
        # put the daq into the right state ('connected')
        logging.debug('*** here in unstage')
        self._set_connected()

    def trigger(self, *, phase1Info=None):
        # do one step
        logging.debug('*** here in trigger')
        if phase1Info is None:
            # BeginStep
            self.push_socket.send_string('running')
        else:
            logging.debug('*** phase1Info = %s' % json.dumps(phase1Info))
            # BeginStep
            self.push_socket.send_string('running,%s' % json.dumps(phase1Info))

        # EndStep
        self.ready.wait()
        self.ready.clear()
        self.push_socket.send_string('starting')
        self.ready.wait()
        self.ready.clear()

    def report_error(self, msg):
        logging.error(msg)
        return

def scan( keys, steps, setupStep=None ):
    parser = argparse.ArgumentParser()
    parser.add_argument('-B', metavar='PVBASE', required=True, help='PV base')
    parser.add_argument('-p', type=int, choices=range(0, 8), default=0,
                        help='platform (default 0)')
    parser.add_argument('-x', metavar='XPM', type=int, required=True, help='master XPM')
    parser.add_argument('-C', metavar='COLLECT_HOST', default='localhost',
                        help='collection host (default localhost)')
    parser.add_argument('-t', type=int, metavar='TIMEOUT', default=10000,
                        help='timeout msec (default 10000)')
    parser.add_argument('-c', type=int, metavar='READOUT_COUNT', default=1, help='# of events to aquire at each step (default 1)')
    parser.add_argument('-g', type=int, metavar='GROUP_MASK', help='bit mask of readout groups (default 1<<plaform)')
    parser.add_argument('--config', metavar='ALIAS', help='configuration alias (e.g. BEAM)')
    parser.add_argument('--record', type=int, choices=range(0, 2), help='recording flag')
    parser.add_argument('-v', action='store_true', help='be verbose')
    args = parser.parse_args()

    if args.g is not None:
        if args.g < 1 or args.g > 255:
            parser.error('readout group mask (-g) must be 1-255')

    if args.c < 1:
        parser.error('readout count (-c) must be >= 1')

    # instantiate DaqControl object
    control = DaqControl(host=args.C, platform=args.p, timeout=args.t)

    try:
        instrument = control.getInstrument()
    except KeyboardInterrupt:
        instrument = None

    if instrument is None:
        sys.exit('Error: failed to read instrument name (check -C <COLLECT_HOST>)')

    # configure logging handlers
    if args.v:
        level=logging.DEBUG
    else:
        level=logging.WARNING
    logging.basicConfig(level=level)
    logging.info('logging initialized')

    # get initial DAQ state
    daqState = control.getState()
    logging.info('initial state: %s' % daqState)
    if daqState == 'error':
        sys.exit(1)

    # optionally set BEAM or NOBEAM
    if args.config is not None:
        # config alias request
        rv = control.setConfig(args.config)
        if rv is not None:
            logging.error('%s' % rv)

    if args.record is not None:
        # recording flag request
        if args.record == 0:
            rv = control.setRecord(False)
        else:
            rv = control.setRecord(True)
        if rv is not None:
            print('Error: %s' % rv)

    # instantiate MyDAQ
    mydaq = MyDAQ(control, daqState=daqState, args=args)

    mydaq.stage()

    # -- begin script --------------------------------------------------------

    configure_dict = {"configure": {"step_keys": keys,
                                    "NamesBlockHex":scanNamesBlock()}}

    for step in steps():
        if setupStep is not None:
            setupStep(step)

        beginstep_dict = {"beginstep": {"step_values": step[0],
                                        "ShapesDataBlockHex":shapesDataBlock(step)}}
        # trigger
        mydaq.trigger(phase1Info = dict(configure_dict, **beginstep_dict))

    # -- end script ----------------------------------------------------------

    mydaq.unstage()

    mydaq.push_socket.send_string('shutdown') #shutdown the daq communicator thread
    mydaq.comm_thread.join()


def setupXtc(step=None):
    d = {'step_value'    :0.,
         'step_docstring':''}
    if step and len(step)==3:
        d['step_value'    ]=step[1]
        d['step_docstring']=step[2]
    print(f'setupXtc {d}')

    nameinfo = dc.nameinfo('scan','scan','1234',253)
    alg      = dc.alg('raw',[2,0,0])
    cydgram  = dc.CyDgram()
    cydgram.addDet(nameinfo, alg, d)
    return cydgram

def scanNamesBlock():
    cydgram = setupXtc()
    return cydgram.getSelect(0, DaqControl.transitionId['Configure'],
                             add_names=True, add_shapes_data=False)[12:].hex()

def shapesDataBlock(step):
    cydgram = setupXtc(step)
    return cydgram.getSelect(0, DaqControl.transitionId['BeginStep'],
                             add_names=False, add_shapes_data=True)[12:].hex()


if __name__ == '__main__':
    main()



import sys
import logging
import threading
from psdaq.control.DaqControl import DaqControl
from psdaq.control.TimedRun import TimedRun
import argparse

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', type=int, choices=range(0, 8), default=2,
                        help='platform (default 2)')
    parser.add_argument('-C', metavar='COLLECT_HOST', default='drp-neh-ctl001',
                        help='collection host (default drp-neh-ctl001)')
    parser.add_argument('-t', type=int, metavar='TIMEOUT', default=10000,
                        help='timeout msec (default 10000)')
    parser.add_argument('-v', action='store_true', help='be verbose')
    parser.add_argument('--duration', type=int, default=10,
                        help='run duration seconds (default 10)')
    args = parser.parse_args()

    # instantiate DaqControl object
    control = DaqControl(host=args.C, platform=args.p, timeout=args.t)

    # configure logging handlers
    if args.v:
        level=logging.DEBUG
    else:
        level=logging.WARNING
    logging.basicConfig(level=level)
    logging.info('logging initialized')

    # get initial DAQ state
    daqState = control.getState()
    logging.info('initial state: %s' % daqState)
    if daqState == 'error':
        sys.exit('failed to get initial DAQ state')

    # instantiate TimedRun
    run = TimedRun(control, daqState=daqState, args=args)

    run.stage()

    try:

        # -- begin script --------------------------------------------------------

        # run daq for the specified time duration
        run.set_running_state()
        run.sleep(args.duration)

        # -- end script ----------------------------------------------------------

    except KeyboardInterrupt:
        run.push_socket.send_string('shutdown') #shutdown the daq communicator thread
        sys.exit('interrupted')

    run.unstage()

    run.push_socket.send_string('shutdown') #shutdown the daq communicator thread

if __name__ == '__main__':
    main()



#!/usr/bin/env python
'''
Get/start/stop the current run for the specified experiment.
'''
import os
import sys
import json
import requests
from requests.auth import HTTPBasicAuth
import argparse

def main():

    # process arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("experiment", help="experiment name")
    parser.add_argument("--user", default="xppopr")
    parser.add_argument("--password", default="pcds")
    parser.add_argument("--start", action="store_true", help="start run")
    parser.add_argument("--end", action="store_true", help="end run")
    defaultURL = "https://pswww.slac.stanford.edu/ws-auth/devlgbk/"
    parser.add_argument("--rundb", metavar='URL', help="URL to post to; this is only the prefix. Defaults to " + defaultURL, default=defaultURL)
    parser.add_argument("--verbose", "-v", action="count", default=0, help="verbose (may be repeated)")
    args = parser.parse_args()

    experiment_name = args.experiment

    serverURLPrefix = "{0}run_control/{1}/ws/".format(args.rundb + "/" if not args.rundb.endswith("/") else args.rundb, experiment_name)

    if args.verbose:
        print('serverURLPrefix = %s' % serverURLPrefix)

    if args.start:
        # start run
        if args.verbose:
            print("Start a run (start_run)")
        try:
            resp = requests.post(serverURLPrefix + "start_run", auth=HTTPBasicAuth(args.user, args.password)  )
        except Exception as ex:
            print('start_run error')
            if args.verbose >= 2:
                print('HTTP request: %s' % ex)
        else:
            if args.verbose >= 2:
                print("Response: %s" % resp.text)
            if resp.json().get("success", None):
                print("start_run success")
            else:
                print("start_run failure")

    # get current run
    if args.verbose:
        print("Get the current run (current_run)")
    try:
        resp = requests.get(serverURLPrefix + "current_run?skipClosedRuns=true", auth=HTTPBasicAuth(args.user, args.password)  )
    except Exception as ex:
        print('current_run error')
        if args.verbose >= 2:
            print('HTTP request: %s' % ex)
    else:
        if args.verbose >= 2:
            print("Response: %s" % resp.text)
        if resp.status_code == requests.codes.ok:
            if resp.json().get("success", None):
                num = resp.json().get("value", {}).get("num", None)
                print("run number: %s" % num)
            else:
                print("current_run failure")
        else:
            exit("Error: status code %d" % resp.status_code)

    if args.end:
        # end run
        if args.verbose:
            print("End a run (end_run)")
        try:
            resp = requests.post(serverURLPrefix + "end_run", auth=HTTPBasicAuth(args.user, args.password)  )
        except Exception as ex:
            print('end_run error')
            if args.verbose >= 2:
                print('HTTP request: %s' % ex)
        else:
            if args.verbose >= 2:
                print("Response: %s" % resp.text)
            if resp.json().get("success", None):
                print("end_run success")
            else:
                print("end_run failure")

if __name__ == '__main__':
    main()



import sys
import logging
import threading
from psdaq.control.ControlDef import ControlDef, MyFloatPv, MyStringPv
from psdaq.control.DaqControl import DaqControl
from psdaq.control.ConfigScan import ConfigScan
import argparse

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', type=int, choices=range(0, 8), default=0,
                        help='platform (default 0)')
    parser.add_argument('-C', metavar='COLLECT_HOST', default='localhost',
                        help='collection host (default localhost)')
    parser.add_argument('-t', type=int, metavar='TIMEOUT', default=10000,
                        help='timeout msec (default 10000)')
    parser.add_argument('-c', type=int, metavar='READOUT_COUNT', default=1, help='# of events to aquire at each step (default 1)')
    parser.add_argument('-g', type=int, metavar='GROUP_MASK', help='bit mask of readout groups (default 1<<platform)')
    parser.add_argument('--config', metavar='ALIAS', help='configuration alias (e.g. BEAM)')
    parser.add_argument('--detname', default='scan', help="detector name (default 'scan')")
    parser.add_argument('--scantype', default='scan', help="scan type (default 'scan')")
    parser.add_argument('-v', action='store_true', help='be verbose')
    args = parser.parse_args()

    if args.g is not None:
        if args.g < 1 or args.g > 255:
            parser.error('readout group mask (-g) must be 1-255')
        group_mask = args.g
    else:
        group_mask = 1 << args.p

    if args.c < 1:
        parser.error('readout count (-c) must be >= 1')

    # instantiate DaqControl object
    control = DaqControl(host=args.C, platform=args.p, timeout=args.t)

    try:
        instrument = control.getInstrument()
    except KeyboardInterrupt:
        instrument = None

    if instrument is None:
        sys.exit('Error: failed to read instrument name (check -C <COLLECT_HOST>)')

    # configure logging handlers
    if args.v:
        level=logging.DEBUG
    else:
        level=logging.WARNING
    logging.basicConfig(level=level)
    logging.info('logging initialized')

    # get initial DAQ state
    daqState = control.getState()
    logging.info('initial state: %s' % daqState)
    if daqState == 'error':
        sys.exit(1)

    # optionally set BEAM or NOBEAM
    if args.config is not None:
        # config alias request
        rv = control.setConfig(args.config)
        if rv is not None:
            logging.error('%s' % rv)

    # instantiate ConfigScan
    scan = ConfigScan(control, daqState=daqState, args=args)

    scan.stage()

    # -- begin script --------------------------------------------------------

    # PV scan setup
    motors = [MyFloatPv(ControlDef.STEP_VALUE)]
    scan.configure(motors = motors)

    my_config_data = {}
    for motor in scan.getMotors():
        my_config_data.update({motor.name: motor.position})
        # derive step_docstring from step_value
        if motor.name == ControlDef.STEP_VALUE:
            docstring = f'{{"detname": "{args.detname}", "scantype": "{args.scantype}", "step": {motor.position}}}'
            my_config_data.update({'step_docstring': docstring})

    data = {
      "motors":           my_config_data,
      "timestamp":        0,
      "detname":          args.detname,
      "dettype":          "scan",
      "scantype":         args.scantype,
      "serial_number":    "1234",
      "alg_name":         "raw",
      "alg_version":      [1,0,0]
    }

    configureBlock = scan.getBlock(transition="Configure", data=data)

    # config scan setup
    keys_dict = {"configure": {"step_keys":     ["tmoopal_0:user.black_level"],
                               "NamesBlockHex": configureBlock},
                 "enable":    {"readout_count": args.c,
                               "group_mask":    group_mask}}
    # scan loop
    for black_level in [15, 31, 47]:
        # update
        scan.update(value=scan.step_count())

        my_step_data = {}
        for motor in scan.getMotors():
            my_step_data.update({motor.name: motor.position})
            # derive step_docstring from step_value
            if motor.name == ControlDef.STEP_VALUE:
                docstring = f'{{"detname": "{args.detname}", "scantype": "{args.scantype}", "step": {motor.position}}}'
                my_step_data.update({'step_docstring': docstring})

        data["motors"] = my_step_data

        beginStepBlock = scan.getBlock(transition="BeginStep", data=data)
        values_dict = \
          {"beginstep": {"step_values":        {"tmoopal_0:user.black_level": black_level},
                         "ShapesDataBlockHex": beginStepBlock}}
        # trigger
        scan.trigger(phase1Info = {**keys_dict, **values_dict})

    # -- end script ----------------------------------------------------------

    scan.unstage()

    scan.push_socket.send_string('shutdown') #shutdown the daq communicator thread
    scan.comm_thread.join()

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
Test File Report
"""

from datetime import datetime, timezone
import zmq
from psdaq.control.control import back_pull_port, create_msg
import argparse

def fileReport_msg(path):
    dt = datetime.now(timezone.utc)
    dts = dt.strftime("%Y-%m-%dT%H:%M:%SZ")
    body = {'path': path,
            'create_timestamp': dts,
            'modify_timestamp': dts,
            'size':0}
    return create_msg('fileReport', body=body)

class Client:
    def __init__(self, platform, collectHost, path):

        # configure zmq sockets
        self.context = zmq.Context(1)
        self.push = self.context.socket(zmq.PUSH)
        self.push.connect('tcp://%s:%d' % (collectHost, back_pull_port(platform)))

        # send fileReport message
        self.push.send_json(fileReport_msg(path))

def main():

    try:
        # process arguments
        parser = argparse.ArgumentParser()
        parser.add_argument('-p', type=int, choices=range(0, 8), default=0, help='platform (default 0)')
        parser.add_argument('-C', metavar='COLLECT_HOST', default='localhost', help='collection host (default localhost)')
        parser.add_argument('-P', metavar='PATH', required=True, help='filename to report')
        args = parser.parse_args()

        # start client
        client = Client(args.p, args.C, args.P)

    except KeyboardInterrupt:
        print('KeyboardInterrupt')

if __name__ == '__main__':
    main()



import logging
import zmq
from threading import Thread, Event, Condition
from psdaq.control.ControlDef import ControlDef, front_pub_port, front_rep_port, create_msg
import time

class TimedRun:
    def __init__(self, control, *, daqState, args):
        self.control = control
        self.name = 'mydaq'
        self.context = zmq.Context()
        self.push_socket = self.context.socket(zmq.PUSH)
        self.push_socket.bind('inproc://timed_run')
        self.pull_socket = self.context.socket(zmq.PULL)
        self.pull_socket.connect('inproc://timed_run')
        self.comm_thread = Thread(target=self.daq_communicator_thread, args=(), daemon=True)
        self.mon_thread = Thread(target=self.daq_monitor_thread, args=(), daemon=True)
        self.ready = Event()
        self.daqState = daqState
        self.args = args
        self.daqState_cv = Condition()
        self.comm_thread.start()
        self.mon_thread.start()
        self.verbose = args.v

    # this thread tells the daq to go to a state and waits for the completion
    def daq_communicator_thread(self):
        logging.debug('*** daq_communicator_thread')
        while True:
            sss = self.pull_socket.recv().decode("utf-8")
            if ',' in sss:
                state, phase1 = sss.split(',', maxsplit=1)
            else:
                state, phase1 = sss, None

            logging.debug('*** received %s' % state)
            if state in ControlDef.states:
                # send 'daqstate(state)' and wait for complete
                errMsg = self.control.setState(state)
                if errMsg is not None:
                    logging.error('%s' % errMsg)
                    continue

                with self.daqState_cv:
                    while self.daqState != state:
                        logging.debug('daqState \'%s\', waiting for \'%s\'...' % (self.daqState, state))
                        self.daqState_cv.wait(1.0)
                        # check for shutdown with nonblocking read
                        try:
                            ttt = self.pull_socket.recv(flags=zmq.NOBLOCK).decode("utf-8")
                        except Exception as ex:
                            pass
                        else:
                            if ttt=='shutdown':
                                return

                    logging.debug('daqState \'%s\'' % self.daqState)

                if self.daqState == state:
                    self.ready.set()

            elif state=='shutdown':
                break

            else:
                logging.error(f'daq_communicator_thread unrecognized input: \'{state}\'')

    def daq_monitor_thread(self):
        logging.debug('*** daq_monitor_thread')
        while True:
            part1, part2, part3, part4, part5, part6, part7, part8 = self.control.monitorStatus()
            if part1 is None:
                break
            elif part1 == 'error':
                logging.error(f"{part2}")
            elif part1 == 'warning':
                logging.warning(f"{part2}")
            elif part1 not in ControlDef.transitions:
                continue

            # part1=transition, part2=state, part3=config
            with self.daqState_cv:
                self.daqState = part2
                self.daqState_cv.notify()

    def sleep(self, secs):
        logging.debug(f'begin {secs} second wait')
        time.sleep(secs)
        logging.debug(f'end {secs} second wait')

    def set_connected_state(self):
        self.push_socket.send_string('connected')
        # wait for complete
        self.ready.wait()
        self.ready.clear()

    def set_running_state(self):
        self.push_socket.send_string('running')
        # wait for complete
        self.ready.wait()
        self.ready.clear()

    def stage(self):
        # done once at start of scan
        # put the daq into the right state ('connected')
        self.set_connected_state()

    def unstage(self):
        # done once at end of scan
        # put the daq into the right state ('connected')
        self.set_connected_state()




import logging
import zmq
from threading import Thread, Event, Condition
import json as oldjson
from psdaq.control.ControlDef import ControlDef, front_pub_port, front_rep_port, create_msg

class ConfigScan:
    def __init__(self, control, *, daqState, args):
        self.control = control
        self.name = 'mydaq'
        self.parent = None
        self.context = zmq.Context()
        self.push_socket = self.context.socket(zmq.PUSH)
        self.push_socket.bind('inproc://config_scan')
        self.pull_socket = self.context.socket(zmq.PULL)
        self.pull_socket.connect('inproc://config_scan')
        self.comm_thread = Thread(target=self.daq_communicator_thread, args=())
        self.mon_thread = Thread(target=self.daq_monitor_thread, args=(), daemon=True)
        self.ready = Event()
        self.step_done = Event()
        self.daqState = daqState
        self.args = args
        self.daqState_cv = Condition()
        self.stepDone_cv = Condition()
        self.stepDone = 0
        self.comm_thread.start()
        self.mon_thread.start()
        self.verbose = args.v
        self.motors = []                # set in configure()
        self._step_count = 0
        self.detname = args.detname
        self.scantype = args.scantype

        if args.g is None:
            self.groupMask = 1 << args.p
        else:
            self.groupMask = args.g

    # this thread tells the daq to do a step and waits for the completion
    def daq_communicator_thread(self):
        logging.debug('*** daq_communicator_thread')
        while True:
            sss = self.pull_socket.recv().decode("utf-8")
            if ',' in sss:
                state, phase1 = sss.split(',', maxsplit=1)
            else:
                state, phase1 = sss, None

            logging.debug('*** received %s' % state)
            if state in ('connected', 'starting'):
                # send 'daqstate(state)' and wait for complete
                if phase1 is None:
                    errMsg = self.control.setState(state)
                else:
                    errMsg = self.control.setState(state, oldjson.loads(phase1))

                if errMsg is not None:
                    logging.error('%s' % errMsg)
                    continue

                with self.daqState_cv:
                    while self.daqState != state:
                        logging.debug('daqState \'%s\', waiting for \'%s\'...' % (self.daqState, state))
                        self.daqState_cv.wait(1.0)
                    logging.debug('daqState \'%s\'' % self.daqState)

                self.ready.set()

            elif state=='running':
                # launch the step with 'daqstate(running)' (with the
                # scan values for the daq to record to xtc2).

                # set DAQ state
                if phase1 is None:
                    errMsg = self.control.setState(state)
                else:
                    errMsg = self.control.setState(state, oldjson.loads(phase1))
                if errMsg is not None:
                    logging.error('%s' % errMsg)
                    continue

                # wait for running
                with self.daqState_cv:
                    while self.daqState != 'running':
                        logging.debug('daqState \'%s\', waiting for \'running\'...' % self.daqState)
                        self.daqState_cv.wait(1.0)
                    logging.debug('daqState \'%s\'' % self.daqState)

                # wait for step done
                logging.debug('Waiting for step done...')
                self.step_done.wait()
                self.step_done.clear()
                logging.debug('step done.')

            elif state=='shutdown':
                break

    def daq_monitor_thread(self):
        logging.debug('*** daq_monitor_thread')
        while True:
            part1, part2, part3, part4, part5, part6, part7, part8 = self.control.monitorStatus()
            if part1 is None:
                break
            elif part1 == 'step':
                self.step_done.set()
                continue
            elif part1 not in ControlDef.transitions:
                continue

            # part1=transition, part2=state, part3=config
            with self.daqState_cv:
                self.daqState = part2
                self.daqState_cv.notify()

    def _set_connected(self):
        self.push_socket.send_string('connected')
        # wait for complete
        self.ready.wait()
        self.ready.clear()

    def stage(self):
        # done once at start of scan
        # put the daq into the right state ('connected')
        self._set_connected()
        self._step_count = 0

    def unstage(self):
        # done once at end of scan
        # put the daq into the right state ('connected')
        logging.debug('*** unstage: step count = %d' % self._step_count)
        self._set_connected()

    # use 'motors' keyword arg to specify a set of motors
    def configure(self, *args, **kwargs):
        logging.debug("*** here in configure")

        if 'motors' in kwargs:
            self.motors = kwargs['motors']
            logging.info('configure: %d motors' % len(self.motors))
        else:
            logging.error('configure: no motors')

    def getMotors(self):
        return self.motors

    def step_count(self):
        return self._step_count

    def update(self, *, value):
        # update 'motors'
        for motor in self.motors:
            motor.update(value)

    def trigger(self, *, phase1Info=None):
        # do one step
        logging.debug('*** trigger: step count = %d' % self._step_count)
        if phase1Info is None:
            phase1Info = {}
        if "beginstep" not in phase1Info:
            phase1Info.update({"beginstep": {}})
        if "configure" not in phase1Info:
            phase1Info.update({"configure": {}})
        if "step_keys" not in phase1Info["configure"]:
            phase1Info["configure"].update({"step_keys": []})
        if "step_values" not in phase1Info["beginstep"]:
            phase1Info["beginstep"].update({"step_values": {}})

        logging.debug('*** phase1Info = %s' % oldjson.dumps(phase1Info))
        # BeginStep
        self.push_socket.send_string('running,%s' % oldjson.dumps(phase1Info))
        # EndStep
        self.push_socket.send_string('starting')
        self._step_count += 1

#
# data = {
#   "motors":           {"motor1": 0.0, "step_value": 0.0},
#   "transition":       "Configure",
#   "timestamp":        0,
#   "add_names":        True,
#   "add_shapes_data":  False,
#   "detname":          "scan",
#   "dettype":          "scan",
#   "scantype":         "scan",
#   "serial_number":    "1234",
#   "alg_name":         "raw",
#   "alg_version":      [2,0,0]
# }
#

    def getBlock(self, *, transition, data):
        logging.debug('getBlock: motors=%s' % data["motors"])
        if transition in ControlDef.transitionId.keys():
            data["transitionid"] = ControlDef.transitionId[transition]
        else:
            logging.error(f'invalid transition: {transition}')

        if transition == "Configure":
            data["add_names"] = True
            data["add_shapes_data"] = False
        else:
            data["add_names"] = False
            data["add_shapes_data"] = True

        data["namesid"] = ControlDef.STEPINFO

        return self.control.getBlock(data)



#!/usr/bin/env python

"""
Test Client
"""

import os
import time
import copy
import socket
import zmq
from psdaq.control.control import back_pull_port, back_pub_port, create_msg
import argparse
import logging
from psdaq.control.syslog import SysLog
import zmq.utils.jsonapi as json

class Client:
    def __init__(self, platform, collectHost, alias):

        # initialize state
        self.state = 'reset'

        # establish id
        self.hostname = socket.gethostname()
        self.pid = os.getpid()
        self.id = hash(self.hostname+str(self.pid))
        self.alias = alias

        # configure zmq sockets
        self.context = zmq.Context(1)
        self.push = self.context.socket(zmq.PUSH)
        self.sub = self.context.socket(zmq.SUB)
        self.push.connect('tcp://%s:%d' % (collectHost, back_pull_port(platform)))
        self.sub.connect('tcp://%s:%d' % (collectHost, back_pub_port(platform)))
        self.sub.setsockopt(zmq.SUBSCRIBE, b'')

        # define commands
        handle_request = {
            'reset': self.handle_reset,
            'rollcall': self.handle_rollcall,
            'alloc': self.handle_alloc,
            'connect': self.handle_connect,
            'disconnect': self.handle_disconnect,
            'configure': self.handle_configure,
            'beginrun': self.handle_beginrun,
            'beginstep': self.handle_beginstep,
            'enable': self.handle_enable,
            'disable': self.handle_disable,
            'endstep': self.handle_endstep,
            'endrun': self.handle_endrun,
            'unconfigure': self.handle_unconfigure
        }

        # process messages
        while True:
            try:
                topic, rawmsg = self.sub.recv_multipart()
            except Exception as ex1:
                logging.error('recv_multipart() exception: %s' % ex1)
            else:
                logging.debug('topic=<%s>' % topic)
                try:
                    msg = json.loads(rawmsg)
                except Exception as ex2:
                    logging.error('json.loads() exception: %s' % ex2)
                else:
                    key = msg['header']['key']
                    handle_request[key](msg)

    def handle_rollcall(self, msg):
        logging.debug('Client handle_rollcall(msg_id=\'%s\')' % msg['header']['msg_id'])
        # time.sleep(1.5)
        body = {'drp': {'proc_info': {
                        'alias': self.alias,
                        'host': self.hostname,
                        'pid': self.pid}}}
        reply = create_msg('rollcall', msg['header']['msg_id'], self.id, body=body)
        self.push.send_json(reply)

    def handle_alloc(self, msg):
        logging.debug('Client handle_alloc(msg_id=\'%s\')' % msg['header']['msg_id'])
        body = {'drp': {'connect_info': {'infiniband': '123.456.789'}}}
        reply = create_msg('alloc', msg['header']['msg_id'], self.id, body)
        self.push.send_json(reply)
        self.state = 'allocated'

    def handle_connect(self, msg):
        logging.debug('Client handle_connect(msg_id=\'%s\')' % msg['header']['msg_id'])
        if self.state == 'allocated':
            self.state = 'connected'
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_disconnect(self, msg):
        logging.debug('Client handle_disconnect(msg_id=\'%s\')' % msg['header']['msg_id'])
        if self.state == 'connected':
            self.state = 'allocated'
            body = {'err_info': 'This is only a test'}
            reply = create_msg('disconnect', msg['header']['msg_id'], self.id, body)
            self.push.send_json(reply)

    def handle_configure(self, msg):
        logging.debug('Client handle_configure(msg_id=\'%s\')' % msg['header']['msg_id'])
        if self.state == 'connected':
            self.state = 'paused'
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_unconfigure(self, msg):
        logging.debug('Client handle_unconfigure(msg_id=\'%s\')' % msg['header']['msg_id'])
        if self.state == 'paused':
            self.state = 'connected'
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_enable(self, msg):
        logging.debug('Client handle_enable(msg_id=\'%s\')' % msg['header']['msg_id'])
        if self.state == 'paused':
            self.state = 'running'
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_disable(self, msg):
        logging.debug('Client handle_disable(msg_id=\'%s\')' % msg['header']['msg_id'])
        if self.state == 'running':
            self.state = 'paused'
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_beginstep(self, msg):
        logging.debug('Client handle_beginstep(msg_id=\'%s\')' % msg['header']['msg_id'])
        if True:
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_endstep(self, msg):
        logging.debug('Client handle_endstep(msg_id=\'%s\')' % msg['header']['msg_id'])
        if True:
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_beginrun(self, msg):
        logging.debug('Client handle_beginrun(msg_id=\'%s\')' % msg['header']['msg_id'])
        if True:
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_endrun(self, msg):
        logging.debug('Client handle_endrun(msg_id=\'%s\')' % msg['header']['msg_id'])
        if True:
            reply = create_msg('ok', msg['header']['msg_id'], self.id)
            self.push.send_json(reply)

    def handle_reset(self, msg):
        logging.debug('Client handle_reset(msg_id=\'%s\')' % msg['header']['msg_id'])
        self.state = 'reset'
        # is a reply to reset necessary?

def main():

    try:
        # process arguments
        parser = argparse.ArgumentParser()
        parser.add_argument('-p', type=int, choices=range(0, 8), default=0, help='platform (default 0)')
        parser.add_argument('-C', metavar='COLLECT_HOST', default='localhost', help='collection host (default localhost)')
        parser.add_argument('-P', metavar='INSTRUMENT', default='tst', help='instrument (default tst)')
        parser.add_argument('-u', metavar='ALIAS', required=True, help='unique ID')
        parser.add_argument('-v', action='store_true', help='be verbose')
        args = parser.parse_args()

        # configure logging
        if args.v:
            level = logging.DEBUG
        else:
            level = logging.WARNING
        logger = SysLog(instrument=args.P, level=level)
        logging.info('logging configured')

        # start client
        client = Client(args.p, args.C, args.u)

    except KeyboardInterrupt:
        logging.info('KeyboardInterrupt')

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
daqstate command
"""
from psdaq.control.DaqControl import DaqControl
from psdaq.control.ControlDef import ControlDef
import argparse
import json

def main():

    # process arguments
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', type=int, choices=range(0, 8), default=0,
                        help='platform (default 0)')
    parser.add_argument('-P', metavar='INSTRUMENT', required=True,
                        help='instrument name (required)')
    parser.add_argument('-C', metavar='COLLECT_HOST', default='localhost',
                        help='collection host (default localhost)')
    parser.add_argument('-t', type=int, metavar='TIMEOUT', default=10000,
                        help='timeout msec (default 10000)')
    parser.add_argument('--phase1', metavar='JSON', default=None,
                        help='phase1Info (only use with --state)')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--state', choices=ControlDef.states)
    group.add_argument('--transition', choices=ControlDef.transitions)
    group.add_argument('--monitor', action="store_true")
    group.add_argument('--config', metavar='ALIAS', help='configuration alias')
    group.add_argument('--record', type=int, choices=range(0, 2), help='recording flag')
    group.add_argument('--bypass', type=int, choices=range(0, 2), help='bypass active detectors file flag')
    group.add_argument('-B', action="store_true", help='shortcut for --config BEAM')
    args = parser.parse_args()

    config = None
    if args.config:
        config = args.config
    elif args.B:
        config = "BEAM"

    # instantiate DaqControl object
    control = DaqControl(host=args.C, platform=args.p, timeout=args.t)

    # verify instrument name match
    try:
        instr = control.getInstrument()
    except KeyboardInterrupt:
        instr = None

    if instr is None:
        exit('Error: failed to read instrument name')
    elif instr != args.P:
        exit('Error: instrument name \'%s\' does not match \'%s\'' %
              (args.P, instr))

    if args.state:
        # change the state
        if args.phase1 is None:
            rv = control.setState(args.state)
        else:
            try:
                phase1 = json.loads(args.phase1)
            except Exception as ex:
                exit('Error: failed to parse JSON: %s' % ex)
            rv = control.setState(args.state, phase1)
        if rv is not None:
            print('Error: %s' % rv)

    elif args.transition:
        # transition request
        rv = control.setTransition(args.transition)
        if rv is not None:
            print('Error: %s' % rv)

    elif config:
        # config alias request
        rv = control.setConfig(config)
        if rv is not None:
            print('Error: %s' % rv)

    elif args.record is not None:
        # recording flag request
        if args.record == 0:
            rv = control.setRecord(False)
        else:
            rv = control.setRecord(True)
        if rv is not None:
            print('Error: %s' % rv)

    elif args.bypass is not None:
        # active detectors file bypass flag request
        if args.bypass == 0:
            rv = control.setBypass(False)
        else:
            rv = control.setBypass(True)
        if rv is not None:
            print('Error: %s' % rv)

    elif args.monitor:
        # monitor the status
        while True:
            part1, part2, part3, part4, part5, part6, part7, part8 = control.monitorStatus()
            if part1 is None:
                break
            elif part1 == 'error':
                print('error: %s' % part2)
            elif part1 == 'warning':
                print('warning: %s' % part2)
            elif part1 == 'fileReport':
                print('data file: %s' % part2)
            elif part1 == 'step':
                print('step_done: %d' % part2)
            elif part1 == 'progress':
                print('progress: %s (%d/%d)' % (part2, part3, part4))
            elif part1 in ControlDef.transitions:
                print('transition: %-11s  state: %-11s  config: %s  recording: %s  bypass_activedet: %s  experiment_name: %s  run_number: %d  last_run_number: %d' %\
                      (part1, part2, part3, part4, part5, part6, part7, part8))
            else:
                print('unknown status: %s' % part1)

    else:
        # print current state
        transition, state, config_alias, recording, platform, bypass_activedet, \
            experiment_name, run_number, last_run_number = control.getStatus()
        print('last transition: %s  state: %s  configuration alias: %s  recording: %s  bypass_activedet: %s  experiment_name: %s  run_number: %d  last_run_number: %d' %\
              (transition, state, config_alias, recording, bypass_activedet, experiment_name, run_number, last_run_number))

if __name__ == '__main__':
    main()



from psdaq.control.config_scan import scan
from psdaq.configdb.get_config import *
import numpy as np
import sys

spacing  = 7
detName  = 'epixquad_0'

def pixel_mask(value0,value1,spacing,position_x,position_y):
    ny,nx=352,384;
    out=np.zeros((ny,nx),dtype=np.int)+value0;
    out[position_y::spacing,position_x::spacing]=value1;
    p = np.pad(out,[(0,4),(0,0)],mode='constant')
    a = np.zeros((4,178,192),dtype=np.uint8)
    a[0] = p[:178,:192]
    a[1] = p[:178,192:]
    a[2] = p[178:,:192]
    a[3] = p[178:,192:]
    quad = np.zeros((16,178,192),dtype=np.uint8)
    for i in range(4):
        quad[i::4] = a[i]
    return quad;

def steps():
    d = {}
    metad = {}
    metad['detname'] = detName
    metad['scantype'] = 'chargeinj'
    d[f'{detName}:user.gain_mode'] = 5  # Map
    for a in range(16):
        saci = f'{detName}:expert.EpixQuad.Epix10kaSaci[{a}]'
        d[f'{saci}.atest'] = 1
        d[f'{saci}.test' ] = 1
        d[f'{saci}.Pulser'] = 0xc8
        # d[f'{saci}:PulserSync'] = 1  # with ghost correction
    for trbit in [0,1]:
        for a in range(16):
            saci = f'{detName}:expert.EpixQuad.Epix10kaSaci[{a}]'
            d[f'{saci}.trbit'] = trbit
        for s in range(spacing**2):
            position_x=position%spacing
            position_y=position//spacing;
            pmask = pixel_mask(0,1,spacing,position_x,position_y)
            d[f'{detName}:user.pixel_map'] = pmask.reshape(-1).tolist()
            #  Set the global meta data
            metad['step'] = s+trbit*spacing**2
            metad['injpos'] = [position_y,position_x]

            yield (d, float(s+trbit*spacing**2), json.dumps(metad))

if __name__ == '__main__':

    # default command line arguments
    if len(sys.argv)==1:
        defargs = ['-B','DAQ:UED','-p','0','-x','0','-C','drp-ued-cmp002','-c 2000','--config','BEAM']
        sys.argv.extend(defargs)

    keys = []
    keys.append(f'{detName}:user.gain_mode')
    keys.append(f'{detName}:user.pixel_map')
    for a in range(16):
        saci = f'{detName}:expert.EpixQuad.Epix10kaSaci[{a}]'
        keys.append(f'{saci}.atest')
        keys.append(f'{saci}.test' )
        keys.append(f'{saci}.trbit' )
        keys.append(f'{saci}.Pulser')

    scan(keys, steps)



#!/usr/bin/env python
'''
Determine the active experiment for the specified instrument/station.
'''
import os
import sys
import json
import requests
import argparse

def main():

    # process arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("instrument", help="instrument_name[:station_number]")
    # unauthenticated URL is OK for read-only access
    defaultURL = "https://pswww.slac.stanford.edu/ws/devlgbk/"
    parser.add_argument("--url", help="URL to post to; this is only the prefix. Defaults to " + defaultURL, default=defaultURL)
    parser.add_argument("--verbose", "-v", action="count", default=0, help="verbose (may be repeated)")
    args = parser.parse_args()

    # parse instrument_name[:station_number]
    if ':' in args.instrument:
        instrument_name, station_number = args.instrument.split(':', maxsplit=1)
        try:
            station = int(station_number)
        except ValueError:
            exit("Error: invalid station number '%s'" % station_number)
    else:
        instrument_name = args.instrument
        station = 0

    try:
        resp = requests.get((args.url + "/" if not args.url.endswith("/") else args.url) + "/lgbk/ws/activeexperiment_for_instrument_station",
                            params={"instrument_name": instrument_name, "station": station}, timeout=10)
    except requests.exceptions.RequestException as ex:
        exit("Error: request exception: %s" % ex)

    if args.verbose >= 2:
        print("response: %s" % resp.text)
    experiment_name = None
    if resp.status_code == requests.codes.ok:
        if args.verbose >= 2:
            print("headers: %s" % resp.headers)
        if 'application/json' in resp.headers['Content-Type']:
            try:
                experiment_name = resp.json().get("value", {}).get("name", None)
            except json.decoder.JSONDecodeError:
                exit("Error: failed to decode JSON")
        else:
            exit("Error: failed to receive JSON")
    else:
        exit("Error: status code %d" % resp.status_code)

    if not experiment_name:
        exit("No experiment found for instrument %s:%d" % (instrument_name, station))

    if args.verbose > 0:
        print("Instrument name: %s" % instrument_name)
        print("Station number: %d" % station)
        print("Experiment name: %s" % experiment_name)
    else:
        print("%s:%d %s" % (instrument_name, station, experiment_name))

if __name__ == '__main__':
    main()



# BlueskyScan.py

from bluesky import RunEngine
from ophyd.status import Status
import sys
import logging
import threading
import zmq
import numpy as np

from psdaq.control.ControlDef import ControlDef

class BlueskyScan:
    def __init__(self, control, *, daqState):
        self.control = control
        self.name = 'mydaq'
        self.parent = None
        self.context = zmq.Context()
        self.push_socket = self.context.socket(zmq.PUSH)
        self.push_socket.bind('inproc://bluesky_scan')
        self.pull_socket = self.context.socket(zmq.PULL)
        self.pull_socket.connect('inproc://bluesky_scan')
        self.comm_thread = threading.Thread(target=self.daq_communicator_thread, args=(), daemon=True)
        self.mon_thread = threading.Thread(target=self.daq_monitor_thread, args=(), daemon=True)
        self.ready = threading.Event()
        self.motors = []                        # set in configure()
        self.group_mask = 1 << control.platform # set in configure()
        self.events=1                           # set in configure()
        self.record=False                       # set in configure()
        self.detname='scan'                     # set in configure()
        self.scantype='scan'                    # set in configure()
        self.serial_number='1234'               # set in configure()
        self.alg_name='raw'                     # set in configure()
        self.alg_version=[1,0,0]                # set in configure()
        self.daqState = daqState
        self.daqState_cv = threading.Condition()
        self.comm_thread.start()
        self.mon_thread.start()
        self.step_done = threading.Event()
        self.step_value = 1

        # StepEnd is a cumulative count
        self.readoutCumulative = 0

    def read(self):
        # stuff we want to give back to user running bluesky
        # e.g. how many events we took.  called when trigger status
        # is done&&successful.
        # typically called by trigger_and_read()
        logging.debug('*** here in read')
        return {}

    def describe(self):
        # stuff we want to give back to user running bluesky
        logging.debug('*** here in describe')
        return {}

    # this thread tells the daq to do a step and waits
    # for the completion, so it can set the bluesky status.
    # it is done as a separate thread so we don't block
    # the bluesky event loop.
    def daq_communicator_thread(self):
        logging.debug('*** daq_communicator_thread')
        while True:
            state = self.pull_socket.recv().decode("utf-8")
            logging.debug('*** received %s' % state)
            if state in ('connected', 'starting'):
                # send 'daqstate(state)' and wait for complete
                # we can block here since we are not in the bluesky
                # event loop
                errMsg = self.control.setState(state)
                if errMsg is not None:
                    logging.error('%s' % errMsg)
                    continue
                with self.daqState_cv:
                    while self.daqState != state:
                        logging.debug('daqState \'%s\', waiting for \'%s\'...' % (self.daqState, state))
                        self.daqState_cv.wait(1.0)
                    logging.debug('daqState \'%s\'' % self.daqState)

                if self.daqState == 'connected':
                    rv = self.control.setRecord(self.record)
                    if rv is not None:
                        logging.error('setRecord(%s): %s' % (self.record, rv))

                self.ready.set()
            elif state=='running':
                if len(self.motors) == 0:
                    logging.error('motors not configured')
                # launch the step with 'daqstate(running)' (with the
                # scan values for the daq to record to xtc2).
                # normally should block on "complete" from the daq here.

                # allow user to override step value
                for motor in self.motors:
                    if motor.name == ControlDef.STEP_VALUE:
                        self.step_value = int(motor.position)
                        logging.debug(f'override: step value = {self.step_value}')
                        break

                my_data = {}

                # record step_value and step_docstring
                my_data.update({'step_value': self.step_value})
                docstring = f'{{"detname": "scan", "scantype": "{self.scantype}", "step": {self.step_value}}}'
                my_data.update({'step_docstring': docstring})

                # record motor positions
                for motor in self.motors:
                    if motor.name == ControlDef.STEP_VALUE:
                        continue
                    my_data.update({motor.name: motor.position})

                data = {
                    "motors":           my_data,
                    "timestamp":        0,
                    "detname":          "scan",
                    "dettype":          "scan",
                    "scantype":         self.scantype,
                    "serial_number":    "1234",
                    "alg_name":         "raw",
                    "alg_version":      [1,0,0],
                }

                configureBlock = self.getBlock(transition="Configure", data=data)
                beginStepBlock = self.getBlock(transition="BeginStep", data=data)

                configure_dict = {"NamesBlockHex": configureBlock,
                                  "readout_count": self.events,
                                  "group_mask"   : self.group_mask,
                                  "step_group"   : self.group }

                if self.seq_ctl is not None:
                    configure_dict['seqpv_name'] = self.seq_ctl[0]
                    configure_dict['seqpv_val' ] = self.seq_ctl[1]
                    if len(self.seq_ctl) > 2:
                        configure_dict['seqpv_done'] = self.seq_ctl[2]

                # set DAQ state
                enable_dict = {'readout_count':self.events, 
                               'group_mask':self.group_mask, 
                               'step_group':self.group}

                errMsg = self.control.setState('running',
                    {'configure':   configure_dict,
                     'beginstep':   {'ShapesDataBlockHex':beginStepBlock},
                     'enable':      enable_dict})
                if errMsg is not None:
                    logging.error('%s' % errMsg)
                    continue

                # wait for running
                with self.daqState_cv:
                    while self.daqState != 'running':
                        logging.debug('daqState \'%s\', waiting for \'running\'...' % self.daqState)
                        self.daqState_cv.wait(1.0)
                    logging.debug('daqState \'%s\'' % self.daqState)

                # wait for step done
                logging.debug('Waiting for step done...')
                self.step_done.wait()
                self.step_done.clear()
                logging.debug(f'step {self.step_value} done.')
                self.step_value += 1

                # tell bluesky step is complete
                # this line is needed in ReadableDevice mode to flag completion
                self.status._finished(success=True)
            elif state=='shutdown':
                break

    def daq_monitor_thread(self):
        logging.debug('*** daq_monitor_thread')
        while True:
            part1, part2, part3, part4, part5, part6, part7, part8 = self.control.monitorStatus()
            if part1 is None:
                break
            elif part1 == 'step':
                self.step_done.set()
                continue
            elif part1 not in ControlDef.transitions:
                continue

            # part1=transition, part2=state, part3=config
            if part1 == 'endrun':
                self.step_value = 1
                logging.debug(f'step value reset to {self.step_value}')

            with self.daqState_cv:
                self.daqState = part2
                self.daqState_cv.notify()

    # for the ReadableDevice style
    # this is a co-routine, so shouldn't block
    def trigger(self):
        # do one step
        self.status = Status()
        # tell the control level to do a step in the scan
        # to-do: pass it the motor positions, ideally both
        # requested/measured positions.
        # maybe don't launch the step directly here
        # with a daqstate command, since it would block
        # the event-loop?
        self.push_socket.send_string('running')     # BeginStep
        self.push_socket.send_string('starting')    # EndStep
        return self.status

    def read_configuration(self):
        # done at the first read after a configure
        return {}

    def describe_configuration(self):
        # the metadata for read_configuration()
        return {}

    def configure(self, *, motors=None, group_mask=None, events=None, record=None, detname=None, scantype=None, serial_number=None, alg_name=None, alg_version=None, seq_ctl=None):
        """Set parameters for scan.

        Keyword arguments:
        motors -- list of motors, optional
            Motors with positions to include in the data stream
        group_mask -- int, optional
            Bit mask of readout groups
        events -- int, optional
            Number of events per scan point
        record -- bool, optional
            Enable recording of data
        detname -- str, optional
            Detector name
        scantype -- str, optional
            Scan type
        serial_number -- str, optional
            Serial number
        alg_name -- str, optional
            Algorithm name
        alg_version -- list of 3 version numbers, optional
            Algorithm version
        seq_ctl -- tuple of PV name and value to be set after each enable
            Sequence PV reset control
        """
        logging.debug("*** here in configure")

        if motors is not None:
            if isinstance(motors, list):
                self.motors = motors
                logging.info('configure: %d motors' % len(self.motors))
            else:
                raise TypeError('motors must be of type list')
        if group_mask is not None:
            if isinstance(group_mask, int):
                self.group_mask = group_mask
            else:
                raise TypeError('group_mask must be of type int')
        if events is not None:
            if isinstance(events, int):
                self.events = events
            else:
                raise TypeError('events must be of type int')
        if record is not None:
            if isinstance(record, bool):
                self.record = record
            else:
                raise TypeError('record must be of type bool')
        if detname is not None:
            if isinstance(detname, str):
                self.detname = detname
            else:
                raise TypeError('detname must be of type str')
        if scantype is not None:
            if isinstance(scantype, str):
                self.scantype = scantype
            else:
                raise TypeError('scantype must be of type str')
        if serial_number is not None:
            if isinstance(serial_number, str):
                self.serial_number = serial_number
            else:
                raise TypeError('serial_number must be of type str')
        if alg_name is not None:
            if isinstance(alg_name, str):
                self.alg_name = alg_name
            else:
                raise TypeError('alg_name must be of type str')
        if alg_version is not None:
            if isinstance(alg_version, list):
                self.alg_version = alg_version
            else:
                raise TypeError('alg_version must be of type list')

        self.seq_ctl = None
        if seq_ctl is not None:
            if not isinstance(seq_ctl[0],str):
                raise TypeError('seq_ctl[0] must be of type str')
            if not isinstance(seq_ctl[1],int):
                raise TypeError('seq_ctl[1] must be of type int')
            if len(seq_ctl)>2 and not isinstance(seq_ctl[2],str):
                raise TypeError('seq_ctl[2] must be of type str')
            self.seq_ctl = seq_ctl

        platform = self.control.getPlatform()
        self.group = None
        for v in platform['drp'].values():
            if (v['active'] == 1 and v['proc_info']['alias'] == detname):
                self.group = v['det_info']['readout']
                break

        if self.group is None:
            self.group = self.control.platform
            logging.warning(f'Readout group not found for {detname}.  Defaulting to platform')
        else:
            logging.info(f'Found readout group {self.group} for {detname}.')

        return (self.read_configuration(),self.read_configuration())

    def _set_connected(self):
        self.push_socket.send_string('connected')
        # wait for complete. is this a coroutine, so we shouldn't block?
        self.ready.wait()
        self.ready.clear()

    def stage(self):
        # done once at start of scan
        # put the daq into the right state ('connected')
        logging.debug('*** here in stage')
        self._set_connected()

        return [self]

    def unstage(self):
        # done once at end of scan
        # put the daq into the right state ('connected')
        logging.debug('*** here in unstage')
        self._set_connected()
        
        return [self]

#
# data = {
#   "motors":           {"motor1": 0.0, "step_value": 0.0},
#   "transition":       "Configure",
#   "timestamp":        0,
#   "add_names":        True,
#   "add_shapes_data":  False,
#   "detname":          "scan",
#   "dettype":          "scan",
#   "scantype":         "scan",
#   "serial_number":    "1234",
#   "alg_name":         "raw",
#   "alg_version":      [2,0,0]
# }
#

    def getBlock(self, *, transition, data):
        logging.debug('getBlock: motors=%s' % data["motors"])
        if transition in ControlDef.transitionId.keys():
            data["transitionid"] = ControlDef.transitionId[transition]
        else:
            logging.error(f'invalid transition: {transition}')

        if transition == "Configure":
            data["add_names"] = True
            data["add_shapes_data"] = False
        else:
            data["add_names"] = False
            data["add_shapes_data"] = True

        data["namesid"] = ControlDef.STEPINFO

        return self.control.getBlock(data)
